﻿using System;
using System.Diagnostics;
using System.Windows.Forms;

namespace EntityFrameworkVisualizer
{
    /// <summary>
    /// Visualizer main form.
    /// </summary>
    public partial class MainForm : Form
    {
        /// <summary>
        /// Web address.
        /// </summary>
        private const string WebAddress = @"http://codeplex.com/site/users/view/kirill_polishchuk";

        /// <summary>
        /// Initializes a new instance of the <see cref="MainForm"/> class.
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Button "Copy" click event handler.
        /// </summary>
        /// <param name="sender">Event sender.</param>
        /// <param name="e">Event data.</param>
        private void ButtonCopyClick(object sender, EventArgs e)
        {
            Clipboard.SetText(string.Join(Environment.NewLine, richTextBox.Lines));
        }

        /// <summary>
        /// Button "Copy" click event handler.
        /// </summary>
        /// <param name="sender">Event sender.</param>
        /// <param name="e">Event data.</param>
        private void ButtonCancelClick(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// Form load event handler.
        /// </summary>
        /// <param name="sender">Event sender.</param>
        /// <param name="e">Event data.</param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            Size = Settings.Default.WindowSize;
            WindowState = Settings.Default.WindowState;
            Location = Settings.Default.Location;
        }

        /// <summary>
        /// Form closing event handler.
        /// </summary>
        /// <param name="sender">Event sender.</param>
        /// <param name="e">Event data.</param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (WindowState != FormWindowState.Maximized)
            {
                Settings.Default.WindowSize = Size;
                Settings.Default.Location = Location;
            }

            Settings.Default.WindowState = WindowState;
            Settings.Default.Save();
        }

        /// <summary>
        /// Link clicked event handler.
        /// </summary>
        /// <param name="sender">Event sender.</param>
        /// <param name="e">Event data.</param>
        private void LinkLabel_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start(WebAddress);
        }
    }
}
